@@ -15,7 +15,7 @@
 module Control.Concurrent.Extra(
     module Control.Concurrent,
     getNumCapabilities, setNumCapabilities, withNumCapabilities,
-    forkFinally, once,
+    forkFinally, once, onceFork,
     -- * Lock
     Lock, newLock, withLock, withLockTry,
     -- * Var
@@ -73,25 +73,42 @@ forkFinally action and_then =
 #endif
 
 
-data Once a = OncePending | OnceRunning (Barrier a) | OnceDone a
-
-
 -- | Given an action, produce a wrapped action that runs at most once.
 --   If the function raises an exception, the same exception will be reraised each time.
+--
+-- > let x ||| y = do t1 <- onceFork x; t2 <- onceFork y; t1; t2
+-- > \(x :: IO Int) -> void (once x) == return ()
+-- > \(x :: IO Int) -> join (once x) == x
+-- > \(x :: IO Int) -> (do y <- once x; y; y) == x
+-- > \(x :: IO Int) -> (do y <- once x; y ||| y) == x
 once :: IO a -> IO (IO a)
 once act = do
     var <- newVar OncePending
-    let run = either throw return
-    return $ join $ modifyVar var $ \v -> case v of
-        OnceDone x -> return (v, run x)
-        OnceRunning x -> return (v, run =<< waitBarrier x)
+    let run = either throwIO return
+    return $ mask $ \unmask -> join $ modifyVar var $ \v -> case v of
+        OnceDone x -> return (v, unmask $ run x)
+        OnceRunning x -> return (v, unmask $ run =<< waitBarrier x)
         OncePending -> do
             b <- newBarrier
             return $ (OnceRunning b,) $ do
-                res <- try_ act
+                res <- try_ $ unmask act
+                signalBarrier b res
                 modifyVar_ var $ \_ -> return $ OnceDone res
                 run res
 
+data Once a = OncePending | OnceRunning (Barrier a) | OnceDone a
+
+
+-- | Like 'once', but immediately starts running the computation on a background thread.
+--
+-- > \(x :: IO Int) -> join (onceFork x) == x
+-- > \(x :: IO Int) -> (do a <- onceFork x; a; a) == x
+onceFork :: IO a -> IO (IO a)
+onceFork act = do
+    bar <- newBarrier
+    forkFinally act $ signalBarrier bar
+    return $ either throwIO return =<< waitBarrier bar
+
 
 ---------------------------------------------------------------------
 -- LOCK
@@ -194,6 +211,7 @@ withVar (Var x) f = withMVar x f
 --   and in some ways is like a manually managed thunk.
 newtype Barrier a = Barrier (Var (Either (MVar ()) a))
     -- Either a Left empty MVar you should wait or a Right result
+    -- With base 4.7 and above readMVar is atomic so you probably can implement Barrier directly on MVar a
 
 -- | Create a new 'Barrier'.
 newBarrier :: IO (Barrier a)
