@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE UnboxedTuples #-}
 
 -- | Extra functions for "Control.Monad".
 --   These functions provide looping, list operations and booleans.
@@ -13,14 +13,13 @@ module Control.Monad.Extra(
     -- * Lists
     partitionM, concatMapM, mapMaybeM, findM, firstJustM,
     -- * Booleans
-    whenM, unlessM, ifM, notM, (||^), (&&^), orM, andM, anyM, allM,
+    whenM, unlessM, ifM, notM, (||^), (&&^), orM, andM, anyM, allM
     ) where
 
 import Control.Monad
-#if __GLASGOW_HASKELL__ < 709
 import Control.Applicative
-#endif
 import Data.Maybe
+import Prelude
 
 -- General utilities
 
@@ -58,11 +57,15 @@ partitionM f (x:xs) = do
 
 -- | A version of 'concatMap' that works with a monadic predicate.
 concatMapM :: Monad m => (a -> m [b]) -> [a] -> m [b]
-concatMapM f = liftM concat . mapM f
+{-# INLINE concatMapM #-}
+concatMapM op = foldr f (return [])
+    where f x xs = do x <- op x; if null x then xs else do xs <- xs; return $ x++xs
 
 -- | A version of 'mapMaybe' that works with a monadic predicate.
 mapMaybeM :: Monad m => (a -> m (Maybe b)) -> [a] -> m [b]
-mapMaybeM f = liftM catMaybes . mapM f
+{-# INLINE mapMaybeM #-}
+mapMaybeM op = foldr f (return [])
+    where f x xs = do x <- op x; case x of Nothing -> xs; Just x -> do xs <- xs; return $ x:xs
 
 -- Looping
 
