@@ -11,7 +11,8 @@ module Data.List.Extra(
     lower, upper, trim, trimStart, trimEnd, word1,
     -- * Splitting    
     dropEnd, takeEnd, splitAtEnd, breakEnd, spanEnd,
-    dropWhileEnd, dropWhileEnd', takeWhileEnd, stripSuffix,
+    dropWhileEnd, dropWhileEnd', takeWhileEnd,
+    stripSuffix, stripInfix, stripInfixEnd,
     wordsBy, linesBy,
     breakOn, breakOnEnd, splitOn, split, chunksOf,
     -- * Basics
@@ -26,11 +27,13 @@ module Data.List.Extra(
     replace, merge, mergeBy,
     ) where
 
+import Control.Applicative
 import Data.List
 import Data.Maybe
 import Data.Function
 import Data.Char
 import Data.Tuple.Extra
+import Prelude
 
 
 -- | Apply some operation repeatedly, producing an element of output
@@ -317,6 +320,7 @@ replace from to [] = []
 -- > breakEnd isLower "youRE" == ("you","RE")
 -- > breakEnd isLower "youre" == ("youre","")
 -- > breakEnd isLower "YOURE" == ("","YOURE")
+-- > \f xs -> breakEnd (not . f) xs == spanEnd f  xs
 breakEnd :: (a -> Bool) -> [a] -> ([a], [a])
 breakEnd f = swap . both reverse . break f . reverse
 
@@ -324,7 +328,8 @@ breakEnd f = swap . both reverse . break f . reverse
 --
 -- > spanEnd isUpper "youRE" == ("you","RE")
 -- > spanEnd (not . isSpace) "x y z" == ("x y ","z")
--- > \f xs-> spanEnd f xs == swap (both reverse (span f (reverse xs)))
+-- > \f xs -> uncurry (++) (spanEnd f xs) == xs
+-- > \f xs -> spanEnd f xs == swap (both reverse (span f (reverse xs)))
 spanEnd :: (a -> Bool) -> [a] -> ([a], [a])
 spanEnd f = breakEnd (not . f)
 
@@ -380,6 +385,7 @@ drop1 (x:xs) = xs
 -- The first element of the returned tuple
 -- is the prefix of @haystack@ before @needle@ is matched.  The second
 -- is the remainder of @haystack@, starting with the match.
+-- If you want the remainder /without/ the patch, use 'stripInfix'.
 --
 -- > breakOn "::" "a::b::c" == ("a", "::b::c")
 -- > breakOn "/" "foobar"   == ("foobar", "")
@@ -398,7 +404,7 @@ breakOn needle (x:xs) = first (x:) $ breakOn needle xs
 --
 -- > breakOnEnd "::" "a::b::c" == ("a::b::", "c")
 breakOnEnd :: Eq a => [a] -> [a] -> ([a], [a])
-breakOnEnd needle haystack = (reverse *** reverse) $ swap $ breakOn (reverse needle) (reverse haystack)
+breakOnEnd needle haystack = both reverse $ swap $ breakOn (reverse needle) (reverse haystack)
 
 
 -- | Break a list into pieces separated by the first
@@ -467,6 +473,27 @@ stripSuffix :: Eq a => [a] -> [a] -> Maybe [a]
 stripSuffix a b = fmap reverse $ stripPrefix (reverse a) (reverse b)
 
 
+-- | Return the the string before and after the search string,
+--   or 'Nothing' if the search string is not present.
+--
+-- Examples:
+--
+-- > stripInfix "::" "a::b::c" == Just ("a", "b::c")
+-- > stripInfix "/" "foobar"   == Nothing
+stripInfix :: Eq a => [a] -> [a] -> Maybe ([a], [a])
+stripInfix needle haystack | Just rest <- stripPrefix needle haystack = Just ([], rest)
+stripInfix needle [] = Nothing
+stripInfix needle (x:xs) = first (x:) <$> stripInfix needle xs
+
+
+-- | Similar to 'stripInfix', but searches from the end of the
+-- string.
+--
+-- > stripInfixEnd "::" "a::b::c" == Just ("a::b", "c")
+stripInfixEnd :: Eq a => [a] -> [a] -> Maybe ([a], [a])
+stripInfixEnd needle haystack = both reverse . swap <$> stripInfix (reverse needle) (reverse haystack)
+
+
 -- | Split a list into chunks of a given size. The last chunk may contain
 --   fewer than n elements. The chunk size must be positive.
 --
