@@ -1,5 +1,6 @@
+{-# LANGUAGE ScopedTypeVariables, CPP #-}
 
-module TestUtil(runTests, testGen, erroneous, (====), module X) where
+module TestUtil(runTests, testGen, testRaw, erroneous, (====), module X) where
 
 import Test.QuickCheck
 import Test.QuickCheck.Test
@@ -22,6 +23,7 @@ import Data.Tuple as X
 import System.Directory as X
 import System.FilePath as X
 import System.Info as X
+import Control.Exception as X
 import Test.QuickCheck as X((==>))
 
 
@@ -30,10 +32,14 @@ testCount :: IORef Int
 testCount = unsafePerformIO $ newIORef 0
 
 testGen :: Testable prop => String -> prop -> IO ()
-testGen msg prop = do
-    putStrLn msg
+testGen msg prop = testRaw msg $ do
     r <- quickCheckResult prop
     unless (isSuccess r) $ error "Test failed"
+
+testRaw :: String -> IO () -> IO ()
+testRaw msg test = do
+    putStrLn msg
+    test
     modifyIORef testCount (+1)
 
 
@@ -43,6 +49,11 @@ erroneous x = unsafePerformIO $ fmap isLeft $ try_ $ evaluate x
 (====) :: (Show a, Eq a) => a -> a -> Bool
 a ==== b = if a == b then True else error $ "Not equal!\n" ++ show a ++ "\n" ++ show b
 
+#if __GLASGOW_HASKELL__ < 707
+instance Eq ErrorCall where
+    ErrorCall x == ErrorCall y = x == y
+#endif
+
 runTests :: IO () -> IO ()
 runTests t = do
     writeIORef testCount 0
@@ -50,6 +61,10 @@ runTests t = do
     n <- readIORef testCount
     putStrLn $ "Success (" ++ show n ++ " tests)"
 
+instance Testable () where
+    property = property . (`seq` True)
+    exhaustive _ = True
+
 instance Testable a => Testable (IO a) where
     property = property . unsafePerformIO
     exhaustive = exhaustive . unsafePerformIO
@@ -60,6 +75,17 @@ instance Eq a => Eq (IO a) where
         b <- try_ $ captureOutput b
         return $ a == b
 
+instance Show (IO a) where
+    show _ = "<<IO>>"
+
+instance Arbitrary a => Arbitrary (IO a) where
+    arbitrary = do
+        (prnt :: Maybe Int, thrw :: Maybe Int, res) <- arbitrary
+        return $ do
+            whenJust prnt print
+            whenJust thrw (fail . show)
+            return res
+
 instance Eq SomeException where
     a == b = show a == show b
 
